How to prepare tasks for BugHunting
===================================

**Usual workflow**
* Download container for testing.
* Prepare the task.
* Check whether the check works.
* Prepare a patch with correct solution for further reference.
* Prepare a pull-requst on gitlab, check CI.

_ _ _ _

**Contents**

[[_TOC_]]

# Steps

## 0. Download container for testing

    $ podman pull docker-registry.upshift.redhat.com/bughunting/devel


Throughout the guide, we'll use this variable:

    $ MY_TASK_NAME=change_me


## 1. Add a new task

Usually, this should be sufficient:

    $ cp -r  warm_up  $MY_TASK_NAME
    $ cd $MY_TASK_NAME
    $ nano task.yml                       # edit task config (nano is a command line editor)
    $ cd source                           # prepare sources
    $ :
    $ cd ../check                         # prepare task check
    $ cd ../..
    $ sudo dnf install perl-YAML-Syck     # install dependency for next step
    $ make check                          # perform basic (static) auto-QA

Task contents:

    warm_up
    ├── check               # check scripts to validate user fix (can be multiple files)
    │   └── check.sh
    ├── fix.patch           # correct fix, more in step 3
    ├── source              # task sources, broken
    │   └── script.sh
    └── task.yml            # task config, please follow instructions in the task.yml file

Don't forget to specify task `dependencies`, and `check_files` in `task.yml`. You can also change the check script to some test suite f.e..


## 2. Check that the task works

Run the container, with mounted tasks directory (run from tasks root directory):

    $ podman run -p 8080:8080 -v "${PWD}:/var/lib/bughunting/tasks:Z,ro" -e MY_TASK_NAME=$MY_TASK_NAME --rm -ti docker-registry.upshift.redhat.com/bughunting/devel

First, 'register' your test user (you won't be able to get points otherwise). **In registration form, input `devel` for Codename (anything as a key).**

    http://127.0.0.1:8080/registration/

_Nothing is sumbmitted out of container. On succesful registration you'll see your task and player name etc.._

Check your tasks summary, description and hints. If you make any edits in your `task.yml`, you need restart the container to be able to view the changes in web interface. If you restart the container, you need to re-register.

_Note: You need to specify dependencies for your task in `task.yaml`, but you also need to install them into the container manually for testing._

**From now on, every command will be run in the container.**

First you need to install dependencies for your task (put those into task.yml as well):

    $ dnf install my-dependency

Next, let's check everything works from user's perspective:

    $ hunt list                         # this should list your task
    $ cd $MY_TASK_NAME
    $ hunt                              # this should fail with:
    Submitting task 'warm_up'.
    ERROR: No score points for this. Try again

Fix the bug, and check it's for points:

    $ nano script.sh                    # fix the bug (nano is a command line editor)
    $ hunt                              # now it should say:
    Submitting task 'warm_up'.
    OK: You gain 1 score points.
    $ hunt patch                        # view the fix, you'll need to save it (step 3)

_Note: this does not make any change outside of container. It works on a copy._

You can also check in the WebUI for the gained points.

In case you've encountered any issues in this step, check the [troubleshooting](#troubleshooting) section.

## 3. Prepare patch with correct solution for further reference

The patch with correct solution will also be helpful in potential discussions whether the user fixed the issue or not (and for CI).

You need to put it into root directory of the task, e.g. `./warm_up/fix.patch`

Run in container from above (step 0):

    $ cd $MY_TASK_NAME
    $ hunt                       # Ensure the task is fixed
    $ hunt patch > fix.patch

You can copy it out like so (uses last container):

    $ cd $MY_TASK_NAME
    $ podman cp `podman ps -l -q`:$MY_TASK_NAME/fix.patch .

_Note: you need to run this out of container._


## 4. Open a pull-request

Read-write access is available via gitlab repository for selected core-services members:

    git@gitlab.cee.redhat.com:bughunting/bhtasks-openhouse-2022.git

It is always preferred (for people without the write access necessary) to **fork
a project on gitlab**, create a pull-request and check CI results before merging.

Before pushing to master, pull latest changes and REBASE them onto your local clone:

    $ git pull --rebase origin master


**Do not produce and push "merge commits" with remote master**, like:

    Merge branch 'master' of https://gitlab.cee.redhat.com/bughunting/bhtasks-openhouse-2022


# Troubleshooting

### Debug output

In case you don't see what you expect (e.g. the file is fixed but you
don't get any points), then it might be worth looking what exactly the server did.

For that, you can use `huntdebug`, instead of `hunt`-  use same commands as above.

    $ cd $MY_TASK_NAME
    $ huntdebug

### Task logs

You can print all server logs for last `hunt` with:

    $ ~/last_log.sh

Alternatively, find the archive directory from the bughunting server that runs locally.
For example, the following:

    $ cd /var/lib/bughunting/archive/9876543210123456_1605078489_bughunting20201111-52-q6a36i
    $ find
    .
    ./work
    ./work/great_task.rb
    ./work/check.sh
    ./work/fix.patch
    ./jail.log
    ./check.log

Although I'd recommend running `hunt` instead, you can run
and debug it from there as well:

    $ cd /var/lib/bughunting/archive/9876543210123456_1605078489_bughunting20201111-52-q6a36i/
    $ chown -R bughunting1:bughunting1 work
    $ chmod -R 0777 work                        # in case owning the dir is not enough
    $ cd work
    $ sudo -u bughunting1 ./check.sh

### Restarting huntserver

If you make some task config changes (or you want to refresh user sources), **you need to restart huntserver**:

    $ ~/restart.sh

_You don't need to restart for any other (code) change._

This restart does not affect the WebUI, so there's no need to re-register.
It also resets all sources, so you need to:

    $ cd "$PWD"

# Contact & References

### References

Everything you need to know: https://docs.google.com/document/d/1GsL4E00vU4FdvMIpYUFuxvow7B4I0r3lfejR9npt_Uw/edit#


### Contact

You can contact us on `#bughunting` on IRC.
If you encounter any issues, please report them in this repo.

### Older tasks for inspiration
* https://gitlab.cee.redhat.com/bughunting/bhtasks-sc2021
* https://gitlab.cee.redhat.com/bughunting/vbhtasks-2021
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2020
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2019
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2018
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2017
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2016
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2015
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2014
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2013
* https://gitlab.cee.redhat.com/bughunting/bhtasks-2012


# Alternatives

## Alternative to container: Install bughunting server and client

If you don't want to download and run the bughunting container, there
is an alternative way, although not recommended, how to develop
tasks for vBughunting.

_Note: bughunting server currently requires root permissions to run, because it's changing users for the task check._

Install 'bughunting-server' and tasks from
https://copr.fedorainfracloud.org/coprs/pvalena/bughunting/]

    $ sudo dnf copr enable pvalena/bughunting
    $ sudo dnf install bughunting-server bughunting-client bughunting bughunting-tasks


Copy, or symlink, your task:

    $ sudo cp -r $MY_TASK_NAME /var/lib/bughunting/tasks/.

or

    $ sudo ln -s `readlink -e $MY_TASK_NAME` /var/lib/bughunting/tasks/$MY_TASK_NAME


Set sources (client) directory, and KEY. And run script to copy tasks into sources directory:

    $ export SOURCES="`mktemp --tmpdir -d bughunting-XXXXX`"
    $ export KEY=9876543210123456
    $ /usr/share/bughunting/scripts/entry-copy-tasks.sh echo

Run servers:

    $ sudo /usr/share/bughunting/scripts/servers_run.rb &>> server.log &

_Note: needs to run as root because of changing user to `bughunting1`._

Then proceed with registration and check same as above (from step 2).

    $ cd ${SOURCES}/$MY_TASK_NAME
    $ hunt


## Deprecated: Using huntlocal to check tasks

As another alternative, same local server installation as above, but different
task dir workflow (less copying).

You need to refer to the [original testing instructions](https://gitlab.cee.redhat.com/bughunting/bhtasks-2020/-/blob/master/README.md#c-check-that-the-task-works).
